@@ -1,17 +1,22 @@
-import json
-from medcat.utils.data_utils import prepare_from_json, encode_category_values, tkns_to_ids
-import torch
-from medcat.utils.ml_utils import train_network
-import numpy as np
-import pickle
 import os
+import json
+import pickle
+import numpy as np
+import torch
+from tokenizers import ByteLevelBPETokenizer
+
+from medcat.utils.ml_utils import train_network
+from medcat.utils.data_utils import prepare_from_json, encode_category_values, tkns_to_ids
 
 class MetaCAT(object):
 
-    def __init__(self, tokenizer, embeddings, cntx_left=20, cntx_right=20,
+    def __init__(self, tokenizer=None, embeddings=None, cntx_left=20, cntx_right=20,
                  save_dir='./meta_cat/', pad_id=30000):
         self.tokenizer = tokenizer
-        self.embeddings = torch.tensor(embeddings, dtype=torch.float32)
+        if embeddings is not None:
+            self.embeddings = torch.tensor(embeddings, dtype=torch.float32)
+        else:
+            self.embeddings = None
         self.cntx_left = cntx_left
         self.cntx_right = cntx_right
         self.save_dir = save_dir
@@ -23,6 +28,10 @@ class MetaCAT(object):
 
         self.model = None
 
+        # TODO: A shitty solution, make right at some point
+        if not self.save_dir.endswith("/"):
+            self.save_dir = self.save_dir + "/"
+
 
     def train(self, json_path, category_name, model_name='lstm', lr=0.01, test_size=0.1,
               batch_size=100, nepochs=20, device='cpu', lowercase=True, class_weights=None, cv=0,
@@ -121,9 +130,15 @@ class MetaCAT(object):
         return inv_map[int(np.argmax(outputs_test.detach().numpy()[0]))]
 
 
-    def save(self):
-        # The model is saved during training, don't do it here
-        #only save the config.
+    def save(self, full_save=False):
+        if full_save:
+            # Save tokenizer and embeddings, slightly redundant
+            self.tokenizer.save(self.save_dir, name='bbpe')
+            # Save embeddings
+            np.save(open(self.save_dir + "embeddings.npy", 'wb'), np.array(self.embeddings))
+
+        # The lstm model is saved during training, don't do it here
+        #save the config.
         self.save_config()
 
 
@@ -132,6 +147,7 @@ class MetaCAT(object):
         to_save = {'category_name': self.category_name,
                    'category_values': self.category_values,
                    'i_category_values': self.i_category_values,
+                   'pad_id': self.pad_id,
                    'cntx_left': self.cntx_left,
                    'cntx_right': self.cntx_right}
         with open(path, 'wb') as f:
@@ -150,11 +166,23 @@ class MetaCAT(object):
         self.i_category_values = to_load['i_category_values']
         self.cntx_left = to_load['cntx_left']
         self.cntx_right = to_load['cntx_right']
+        self.pad_id = to_load.get('pad_id', 0)
 
 
-    def load(self, model='lstm'):
+
+    def load(self, model='lstm', tokenizer_name='bbpe'):
         """ Loads model and config for this meta annotation
         """
+        # Load tokenizer if it is None
+        if self.tokenizer is None:
+            vocab_file = self.save_dir + "{}-vocab.json".format(tokenizer_name)
+            merges_file = self.save_dir + "{}-merges.txt".format(tokenizer_name)
+            self.tokenizer = ByteLevelBPETokenizer(vocab_file=vocab_file, merges_file=merges_file)
+        # Load embeddings if None
+        if self.embeddings is None:
+            embeddings = np.load(open(self.save_dir  + "embeddings.npy", 'rb'))
+            self.embeddings = torch.tensor(embeddings, dtype=torch.float32)
+
         self.load_config()
         # Load MODEL
         if model == 'lstm':
